using System;
using Autofac.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Configuration;
using Serilog;
using MediatR;
using IRaCIS.Core.Application.MediatR.Handlers;
using System.Threading.Tasks;
using MassTransit;
using MassTransit.NewIdProviders;
using System.IO;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Infra.EFCore;
using IRaCIS.Core.Application.Helper;
using System.Runtime.InteropServices;
using Microsoft.AspNetCore.Builder;
using IRaCIS.Core.API;
using Autofac;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.DependencyInjection;
using IRaCIS.Core.Application.Filter;
using Microsoft.AspNetCore.HttpOverrides;
using IRaCIS.Application.Services.BackGroundJob;
using LogDashboard;
using OfficeOpenXml.Utils;



#region ȡ
//ļΪ׼ urlȡֵ(дݲļ˾ͲҪݻ)
var config = new ConfigurationBuilder()
            .AddEnvironmentVariables()
            .Build();

var enviromentName = config["ASPNETCORE_ENVIRONMENT"];

if (string.IsNullOrWhiteSpace(enviromentName))
{

    var index = Array.IndexOf(args, "--env");
    enviromentName = index > -1
        ? args[index + 1]
        : "Development";
}
#endregion

var builder = WebApplication.CreateBuilder(new WebApplicationOptions
{
    EnvironmentName = enviromentName
});

#region windows еķʽ

//foreach (var arg in args)
//{
//    Console.WriteLine(arg);
//}

int urlsIndex = Array.FindIndex(args, arg => arg != null && arg.StartsWith("--urls"));

if (urlsIndex > -1)
{
    var url = args[urlsIndex].Substring("--urls=".Length);
    Console.WriteLine(url);
    builder.WebHost.UseUrls(url);
}

#endregion

#region 


NewId.SetProcessIdProvider(new CurrentProcessIdProvider());



builder.Host
            .ConfigureAppConfiguration((hostContext, config) =>
            {

                //Console.WriteLine(hostContext.HostingEnvironment.EnvironmentName);                
                config.AddJsonFile("appsettings.json", false, true)
                          .AddJsonFile($"appsettings.{enviromentName}.json", false, true);
            })
            .UseServiceProviderFactory(new AutofacServiceProviderFactory())
            .ConfigureContainer<ContainerBuilder>(containerBuilder =>
            {
                containerBuilder.RegisterModule<AutofacModuleSetup>();
            })
            .UseWindowsService().UseSerilog();
#endregion


#region ÷
var _configuration = builder.Configuration;

//
builder.Services.AddHealthChecks();
//ػ
builder.Services.AddJsonLocalization(options => options.ResourcesPath = "Resources");

// 쳣ͳһ֤JsonлáַͳһTrim()
builder.Services.AddControllers(options =>
{
    //options.Filters.Add<LogActionFilter>();
    options.Filters.Add<ModelActionFilter>();
    options.Filters.Add<ProjectExceptionFilter>();
    options.Filters.Add<UnitOfWorkFilter>();

    if (_configuration.GetSection("BasicSystemConfig").GetValue<bool>("OpenLoginLimit"))
    {
        options.Filters.Add<LimitUserRequestAuthorization>();
    }

})
    .AddNewtonsoftJsonSetup(); // NewtonsoftJson л  

builder.Services.AddOptions().Configure<SystemEmailSendConfig>(_configuration.GetSection("SystemEmailSendConfig"));
builder.Services.AddOptions().Configure<ServiceVerifyConfigOption>(_configuration.GetSection("BasicSystemConfig"));
builder.Services.AddOptions().Configure<AliyunOSSOptions>(_configuration.GetSection("AliyunOSS"));
builder.Services.AddOptions().Configure<ObjectStoreServiceOptions>(_configuration.GetSection("ObjectStoreService"));


//̬WebApi + UnifiedApiResultFilter  ʡ
builder.Services.AddDynamicWebApiSetup();
//AutoMapper
builder.Services.AddAutoMapperSetup();
//EF ORM  QueryWithNoLock
builder.Services.AddEFSetup(_configuration);
//Http Ӧѹ
builder.Services.AddResponseCompressionSetup();
//Swagger Api ĵ
builder.Services.AddSwaggerSetup();
//JWT Token ֤
builder.Services.AddJWTAuthSetup(_configuration);

// MediatR Ϣ ¼  ӳ עhandlerӦϵ
builder.Services.AddMediatR(cfg => cfg.RegisterServicesFromAssemblyContaining<ConsistencyVerificationHandler>());
// EasyCaching 
builder.Services.AddEasyCachingSetup(_configuration);

// hangfire  ʱ  н棬Ѻ~
builder.Services.AddhangfireSetup(_configuration);


//Serilog ־ӻ LogDashboard־
builder.Services.AddLogDashboardSetup();


builder.Services.AddJsonConfigSetup(_configuration);
//תͷ  ȡʵIP 
builder.Services.Configure<ForwardedHeadersOptions>(options =>
{
    options.ForwardedHeaders =
        ForwardedHeaders.XForwardedFor | ForwardedHeaders.XForwardedProto;
});
//DicomӰȾͼƬ  ƽ̨
builder.Services.AddDicomSetup();

// ʵʱӦ
builder.Services.AddSignalR();

builder.Services.AddSingleton<IUserIdProvider, IRaCISUserIdProvider>();





#region  ʷ
//builder.Services.AddMemoryCache();
////ϴ 
//builder.Services.Configure<FormOptions>(options =>
//{
//    options.MultipartBodyLengthLimit = int.MaxValue;
//    options.ValueCountLimit = int.MaxValue;
//    options.ValueLengthLimit = int.MaxValue;
//});
//IP  ð ߺ
//services.AddIpPolicyRateLimitSetup(_configuration);
// û Ȩ
//services.AddAuthorizationPolicySetup(_configuration);
#endregion

#endregion

var app = builder.Build();
var env = app.Environment;

#region м


// Configure the HTTP request pipeline.

//ػ
app.UseLocalization();

app.UseForwardedHeaders();

//Ӧѹ
app.UseResponseCompression();

//app.UseCors(t => t.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());

//Ҫ token ʵľ̬ļ  wwwroot css, JavaScript, and images don't require authentication.
app.UseStaticFiles();

app.UseMiddleware<MultiDiskStaticFilesMiddleware>();

//LogDashboard
app.UseLogDashboard("/LogDashboard");

//hangfire
app.UseHangfireConfig(env);


//// м
//app.UseIpRateLimiting();


if (env.IsDevelopment())
{
    app.UseDeveloperExceptionPage();
}
else
{
    //app.UseHsts();
}

// 쳣  404 
app.UseStatusCodePagesWithReExecute("/Error/{0}");

SwaggerSetup.Configure(app, env);


////serilog ¼ûϢ
app.UseSerilogConfig(env);

app.UseRouting();

app.UseCors(t => t.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());

//app.UseIRacisHostStaticFileStore(env);


app.UseAuthentication();
app.UseAuthorization();


app.UseEndpoints(endpoints =>
{
    endpoints.MapControllers();

    endpoints.MapHub<UploadHub>("/UploadHub");

    endpoints.MapHealthChecks("/health");
});

// Serilog 
SerilogExtension.AddSerilogSetup(enviromentName, app.Services);


var hangfireJobService = app.Services.GetRequiredService<IIRaCISHangfireJob>();

await hangfireJobService.InitHangfireJobTaskAsync();


#endregion

try
{
    #region л ƽ̨

    Log.Logger.Warning($"ǰ{enviromentName}");

    if (RuntimeInformation.IsOSPlatform(OSPlatform.Windows))
    {
        Log.Logger.Warning($"ǰƽ̨windows");
    }
    else if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
    {
        Log.Logger.Warning($"ǰƽ̨linux");
    }
    else
    {
        Log.Logger.Warning($"ǰƽ̨OSX  or FreeBSD");
    }

    #endregion


    app.Run();




}
catch (Exception e)
{

    Log.Logger.Error(e.InnerException is null ? e.Message + e.StackTrace : e.InnerException?.Message + e.InnerException?.StackTrace);
}



